# -*- coding: utf-8 -*-
# author: Ethosa

import requests

from ..StartThread import StartThread

from .LongPoll import LongPoll
from .VkAuthManager import VkAuthManager
from .Uploader import Uploader
from .VkScript import VkScript


class Vk(object):
    def __init__(self, token="", group_id="",
                 login="", password="", api="5.103",
                 debug=False):
        """auth in VK

        Keyword Arguments:
            token {str} -- access_token (default: {""})
            group_id {str} -- group id if you want to log in through the group (default: {""})
            login {str} -- login. used for authorization through the user (default: {""})
            password {str} -- password. used for authorization through the user (default: {""})
            api {str} -- api version (default: {"5.103"})
            debug {bool} -- debug log (default: {False})
        """
        self.session = requests.Session()
        self.is_lp = False
        if login and password:
            self.auth = VkAuthManager(self, login, password)
            self.auth.login()
            token = self.auth.get_token()
            self.is_lp = True

        self.v = api
        self.token = token
        self.debug = debug
        self.method = ""
        self.events = {}
        self.group_id = group_id

        self.execute = lambda code: self.call_method("execute", {"code": code})
        self.uploader = Uploader(self)
        self.longpoll = LongPoll(self)
        self.pyexecute = lambda code: self.call_method(
            "execute", {"code": VkScript().translate(code)}
        )

    def call_method(self, method, data={}):
        """call to any method in VK api

        Arguments:
            method {str} -- method name
            e.g. "messages.send", "wall.post"

        Keyword Arguments:
            data {dict} -- data to send (default: {{}})

        Returns:
            {dict} -- response after calling method
        """
        data["v"] = self.v
        data["access_token"] = self.token
        response = self.session.post(
                "https://api.vk.com/method/%s" % method, data=data
            ).json()
        if self.debug:
            if "error" in response:
                print('[DEBUG]: Error [%s] in called method "%s": %s' % (
                        response["error"]["error_code"], method, response["error"]["error_msg"]
                    )
                )
            else:
                print('[DEBUG]: Successfully called method "%s"' % (method))
        return response

    def start_listen(self):
        """starts receiving events from the server
        """
        if not self.longpoll.lend:
            self.longpoll.lend = lambda event: self.start_listen

        if self.debug:
            print("[DEBUG]: On")
            print("[DEBUG]: Started to listen ...")

        for event in self.longpoll.listen(True):
            if "type" in event:
                if "event_%s" % event["type"] in self.events:
                    self.events["event_%s" % event["type"]](event)
                elif event["type"] in dir(self):
                    self.__getattribute__(event["type"])(event)
            elif self.debug:
                print('[DEBUG]: Unknown event passed: "%s"' % (event))

    def __getattr__(self, attr):
        """a convenient alternative for the call_method method

        Arguments:
            attr {str} -- method name
            e.g. messages.send, wall.post

        Returns:
            response after calling method
        """
        if attr.startswith("on_"):
            attr = attr[3:]

            def decorator(func):
                obj_type = "%s" % type(func)

                def listen(f):
                    for event in self.longpoll.listen(True, True):
                        if event["type"] == attr:
                            func(event)
                if "method" in obj_type or "function" in obj_type:
                    StartThread(listen, func).start()
                else:
                    if func:
                        def _decorator(call):
                            StartThread(listen, call).start()
                        return _decorator
                    else:
                        def _decorator(call):
                            self.events["event_%s" % attr] = call
                            return call
                        return _decorator

            return decorator
        elif self.method:
            method = "%s.%s" % (self.method, attr)
            self.method = ""
            return lambda **data: self.call_method(method, data)
        else:
            self.method = attr
            return self



# -*- coding: utf-8 -*-
# author: Ethosa

import json


class Keyboard(dict):
    def __init__(self, one_time=False, inline=False, other_keyboard=None):
        if other_keyboard:
            self.__init__(other_keyboard["one_time"], other_keyboard["inline"])
            self["buttons"] = other_keyboard["buttons"][:]
        else:
            self["inline"] = inline
            self["one_time"] = one_time
            self["buttons"] = [[]]
        if self["inline"]:
            self.max_size = (3, 3)
            del self["one_time"]
        else:
            self.max_size = (4, 10)

    def add(self, button):
        if len(self["buttons"]) < self.max_size[1]:
            if len(self["buttons"][-1]) < self.max_size[0]:
                if button["action"]["type"] == "text":
                    self["buttons"][-1].append(button)
                else:
                    if len(self["buttons"][-1]) > 0:
                        self.add_line()
                    self["buttons"][-1].append(button)

    def add_line(self):
        if len(self["buttons"]) < self.max_size[1]:
            self["buttons"].append([])

    def compile(self):
        return json.dumps(self)



# -*- coding: utf-8 -*-
# author: Ethosa

from .Vk import Vk
from .LongPoll import LongPoll
from .VkAuthManager import VkAuthManager
from .Event import Event
from .VkScript import VkScript
from .Uploader import Uploader
from .Button import Button
from .Keyboard import Keyboard
from .TemplateElement import TemplateElement
from .Template import Template

if __name__ == '__main__':
    print(Vk, LongPoll, VkAuthManager, Event, VkScript,
          Uploader, Button, Keyboard, TemplateElement,
          Template)



# -*- coding: utf-8 -*-
# author: Ethosa

from .Event import Event


class LongPoll:
    def __init__(self, vk):
        """Class for using longpoll in VK

        Arguments:
            vk {Vk} -- authed VK object
        """
        self.session = vk.session
        self.token = vk.token
        self.group_id = vk.group_id
        self.v = vk.v
        self.debug = vk.debug

        self.events = []
        self.opened = 0
        self.lend = lambda arg: None

        if self.group_id:
            self.method = "https://api.vk.com/method/groups.getLongPollServer"
            self.for_server = "%s?act=a_check&key=%s&ts=%s&wait=25"
        else:
            self.method = "https://api.vk.com/method/messages.getLongPollServer"
            self.for_server = "https://%s?act=a_check&key=%s&ts=%s&wait=25&mode=202&version=3"

    def listen(self, event=False, autorestart=False):
        """Start listening

        Yields:
            {dict} -- new event
        """
        self.opened += 1
        data = {
            "access_token": self.token,
            "group_id": self.group_id,
            "v": self.v
        }
        if not self.group_id:
            del data["group_id"]
        response = self.session.get(self.method, params=data).json()["response"]

        server, ts, key = response["server"], response["ts"], response["key"]

        if self.debug:
            print("[DEBUG]: LongPoll launched")

        while 1:
            response = self.session.get(self.for_server % (server, key, ts)).json()
            if "ts" not in response or "updates" not in response:
                if not autorestart:
                    break
                else:
                    response = self.session.get(self.method, params=data).json()["response"]
                    server, ts, key = response["server"], response["ts"], response["key"]
            ts = response["ts"]
            updates = response["updates"]

            for update in updates:
                if update:
                    if event:
                        yield Event(update)
                    else:
                        yield update

            if self.events:
                yield self.events.pop()
        if self.debug:
            print("[DEBUG]: LongPoll has been stopped. trying to call a method ...")
        self.lend(event)

    def on_listen_end(self, call):
        """Sets the function that is called when listening is completed.

        Arguments:
            call {method, function or class} -- callable object

        Returns:
            call
        """
        self.lend = call
        return call

    def push(self, event):
        """Adds a new event to Longpoll

        Arguments:
            event {dict} -- event info. Must contain a "type" key for normal operation
        """
        for _ in range(self.opened):
            self.events.append(event)



# -*- coding: utf-8 -*-
# author: Ethosa


class TemplateElement(dict):
    def __init__(self, title="", description="",
                 photo_id="", buttons=[], action="open_link",
                 link="https://vk.com"):
        self["title"] = title
        self["description"] = description
        self["photo_id"] = photo_id
        self["buttons"] = buttons
        self["action"] = {"type": action}
        if action == "open_link":
            self["action"]["link"] = link

    def add(self, button):
        if len(self["buttons"]) < 3:
            self["buttons"].append(button)



# -*- coding: utf-8 -*-
# author: Ethosa

from threading import Thread


class StartThread(Thread):
    def __init__(self, callable_obj, *args, **kwargs):
        self.callable_obj = callable_obj
        self.args = args
        self.kwargs = kwargs
        Thread.__init__(self)

    def run(self):
        self.callable_obj(*self.args, **self.kwargs)



# -*- coding: utf-8 -*-
# author: Ethosa


class Button(dict):
    PRIMARY = "primary"
    SECONDARY = "secondary"
    POSITIVE = "positive"
    NEGATIVE = "negative"

    def __init__(self, btype="text", color="primary", **kwargs):
        if btype == "text":
            self["color"] = color
        self["action"] = kwargs
        self["action"]["type"] = btype

    def set_color(self, color="primary"):
        self["color"] = color



# -*- coding: utf-8 -*-
# author: Ethosa

import json


class Template(dict):
    def __init__(self, ttype="carousel"):
        self["type"] = ttype
        self["elements"] = []

    def add(self, element):
        """add new element in template

        Arguments:
            element {TemplateElement}
        """
        if len(self["elements"]) < 10:
            self["elements"].append(element)

    def compile(self):
        return json.dumps(self)
